@@ -9,14 +9,14 @@ use tap::Tap;
 
 use crate::{
     containers::{List, Map, Symbol},
-    context::{Ctx, CtxLocation, CtxResult, ModuleId, ToCtx, ToCtxErr},
+    context::{ProjectRoot, Ctx, CtxLocation, CtxResult, ModuleId, ToCtx, ToCtxErr},
     grammar::{BinOp, UniOp},
 };
 
 use super::{RawConstExpr, RawDefn, RawExpr, RawProgram, RawTypeBind, RawTypeExpr};
 
 /// Parse a string as an entire program.
-pub fn parse_program(input: &str, source: ModuleId) -> CtxResult<Ctx<RawProgram>> {
+pub fn parse_program(input: &str, source: ModuleId, root: &Path) -> CtxResult<Ctx<RawProgram>> {
     log::debug!("parsing {} chars at {}", input.len(), source);
     let root_ctx = CtxLocation {
         source,
@@ -55,7 +55,7 @@ pub fn parse_program(input: &str, source: ModuleId) -> CtxResult<Ctx<RawProgram>
     let definitions: List<Ctx<RawDefn>> = children
         .iter()
         .filter(|child| child.as_rule() == Rule::definition)
-        .map(|child| parse_definition(child.clone().into_inner().next().unwrap(), source))
+        .map(|child| parse_definition(child.clone().into_inner().next().unwrap(), source, root))
         .collect();
     let body = if let Some(last_child) = children
         .into_iter()
@@ -68,7 +68,7 @@ pub fn parse_program(input: &str, source: ModuleId) -> CtxResult<Ctx<RawProgram>
     Ok(RawProgram { definitions, body }.with_ctx(Some(ctx)))
 }
 
-fn parse_definition(pair: Pair<Rule>, source: ModuleId) -> Ctx<RawDefn> {
+fn parse_definition(pair: Pair<Rule>, source: ModuleId, root: &Path) -> Ctx<RawDefn> {
     log::trace!("defn rule {:?} on {:?}", pair.as_rule(), pair.as_str());
     match pair.as_rule() {
         Rule::fun_def | Rule::fun_def_gen => {
@@ -88,7 +88,7 @@ fn parse_definition(pair: Pair<Rule>, source: ModuleId) -> Ctx<RawDefn> {
                         Rule::type_name => &mut genvars,
                         _ => unreachable!(),
                     }
-                    .push_back(Symbol::from(elem.as_str()).with_ctx(elem_ctx));
+                    .push(Symbol::from(elem.as_str()).with_ctx(elem_ctx));
                 }
                 (cgvars, genvars)
             } else {
@@ -134,13 +134,16 @@ fn parse_definition(pair: Pair<Rule>, source: ModuleId) -> Ctx<RawDefn> {
         Rule::require_lib => {
             let ctx = p2ctx(&pair, source);
             let children: Vec<_> = pair.into_inner().map(|p| p.as_str().to_string()).collect();
-            RawDefn::Require(source.relative(&children.join("/")))
-            /*
-            RawDefn::Require(ModuleId::from_path(Path::new(&format!(
-                "${}",
-                children.join("/")
-            ))))
-            */
+            let lib_path = &root.join(&children.join("/"));
+            let root = ProjectRoot(root.to_path_buf());
+            if !lib_path.exists() {
+                //println!("{:?}", RawDefn::Require(root.clone().module_from_root(&lib_path.with_extension("melo"))));
+                // Module is a single file
+                RawDefn::Require(root.module_from_root(&lib_path.with_extension("melo")))
+            } else {
+                // Module is a sub-directory
+                RawDefn::Require(root.module_from_root(&lib_path.join("main.melo")))
+            }
             .with_ctx(ctx)
         }
         Rule::provide => {
@@ -171,13 +174,13 @@ fn parse_fun_args(pair: Pair<Rule>, source: ModuleId) -> List<Ctx<RawTypeBind>>
         let var_type = children.next().unwrap();
         let var_type = parse_type_expr(var_type, source);
         let ctx = var_type.ctx();
-        toret.push_back(
+        toret.push(
             RawTypeBind {
                 name: var_name,
                 bind: var_type,
             }
             .with_ctx(ctx),
-        )
+        );
     }
     toret
 }
@@ -313,12 +316,21 @@ fn parse_expr(pair: Pair<Rule>, source: ModuleId) -> Ctx<RawExpr> {
             RawExpr::If(condition, x, RawExpr::Fail.into()).with_ctx(ctx)
         }
         Rule::let_expr => {
-            let mut children = pair.into_inner();
-            let var_name = children.next().unwrap();
-            let var_name = Symbol::from(var_name.as_str()).with_ctx(p2ctx(&var_name, source));
-            let var_binding = parse_expr(children.next().unwrap(), source);
-            let body = parse_expr(children.next().unwrap(), source);
-            RawExpr::Let(var_name, var_binding, body).with_ctx(ctx)
+            let mut children: Vec<_> = pair.into_inner().collect();
+
+            let bindings = children
+                .chunks_exact(2)
+                //.map(|[sym, value]| {
+                .map(|chunk| {
+                    let sym = chunk[0].clone();
+                    let value = chunk[1].clone();
+                    let var_name = Symbol::from(sym.as_str()).with_ctx(p2ctx(&sym, source));
+                    let var_binding = parse_expr(value, source);
+                    (var_name, var_binding)
+                }).collect();
+
+            let body = parse_expr(children.last().unwrap().clone(), source);
+            RawExpr::Let(bindings, body).with_ctx(ctx)
         }
         Rule::let_q_expr => {
             let mut children = pair.into_inner();
@@ -328,7 +340,7 @@ fn parse_expr(pair: Pair<Rule>, source: ModuleId) -> Ctx<RawExpr> {
             let body = parse_expr(children.next().unwrap(), source);
             let body_container =
                 RawExpr::BinOp(BinOp::Land.into(), RawExpr::Var(*var_name).into(), body);
-            RawExpr::Let(var_name, var_binding, body_container.into()).with_ctx(ctx)
+            RawExpr::Let([(var_name, var_binding)].into(), body_container.into()).with_ctx(ctx)
         }
         Rule::for_expr => {
             let mut children = pair.into_inner();
@@ -426,7 +438,7 @@ fn parse_expr(pair: Pair<Rule>, source: ModuleId) -> Ctx<RawExpr> {
                 .with_ctx(p2ctx(&fun_name, source));
             let mut args = List::new();
             for arg in children.next().unwrap().into_inner() {
-                args.push_back(parse_expr(arg, source));
+                args.push(parse_expr(arg, source));
             }
             RawExpr::ExternApply(fun_name, args).with_ctx(ctx)
         }
@@ -538,6 +550,14 @@ fn parse_expr(pair: Pair<Rule>, source: ModuleId) -> Ctx<RawExpr> {
                 .into_iter()
                 .map(|c| parse_setbang(c, source))
                 .collect();
+            /*
+            let free_vars = inner.iter().map(|(sym, val)| val.free_variables());
+
+            RawExpr::Let(
+                free_vars.into_iter().map(|sym| (sym, sym)).collect(),
+                RawExpr::Loop(iterations, inner, end_with).with_ctx(ctx),
+            ).with_ctx(ctx)
+            */
             RawExpr::Loop(iterations, inner, end_with).with_ctx(ctx)
         }
         Rule::string_literal => {
@@ -591,7 +611,8 @@ mod tests {
                     let ctr = 0 :: Nat in
                     accum
             "#,
-                ModuleId::from_path(Path::new("placeholder.melo"))
+                ModuleId::from_path(Path::new("placeholder.melo")),
+                &std::path::PathBuf::from(""),
             )
             .unwrap()
         );
